@@ -35,65 +35,19 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_bsd.c,v 1.12 1997/07/16 19:28:14 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_bsd.c,v 1.20 1997/10/17 17:45:15 kalt Exp $";
 #endif
 
-#include <sys/types.h>
-#include <utmp.h>
-#include "struct.h"
-#include "common.h"
-#include "sys.h"
-#include "res.h"
-#include "numeric.h"
-#include <math.h>
-#include <sys/socket.h>
-#include <sys/file.h>
-#include <sys/ioctl.h>
-#if defined(SVR4)
-#include <sys/filio.h>
-#endif
-#if defined(UNIXPORT) && (!defined(SVR3) || defined(SGI) || \
-    defined(_SEQUENT_))
-# include <sys/un.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-# include <arpa/inet.h>
-#endif
-#include <stdio.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <unistd.h>		/* Linux - mkdir() */
-#include <sys/stat.h>		/* Linux - chmod() */
-#include <sys/resource.h>
-
-#ifdef _DO_POLL_
-#include <stropts.h>
-#include <poll.h>
-#endif /* _DO_POLL_ */
-
-#ifdef HAVE_ARPA_NAMESER_H
-# ifdef	AIX
-#  include <time.h>
-# endif
-# include <arpa/nameser.h>
-#else
-# include "nameser.h"
-#endif /* HAVE_ARPA_NAMESER_H */
-#ifdef HAVE_RESOLV_H
-# include <resolv.h>
-#else
-# include "resolv.h"
-#endif
-#include "sock.h"	/* If FD_ZERO isn't define up to this point,  */
-			/* define it (BSD4.2 needs this) */
-#include "h.h"
+#include "os.h"
+#include "s_defines.h"
+#define S_BSD_C
+#include "s_externs.h"
+#undef S_BSD_C
 
 #ifndef IN_LOOPBACKNET
 #define IN_LOOPBACKNET	0x7f
 #endif
 
-extern  char    serveropts[];
-
 aClient	*local[MAXCONNECTIONS];
 FdAry	fdas, fdaa, fdall;
 int	highest_fd = 0, readcalls = 0, udpfd = -1, resfd = -1;
@@ -121,7 +75,7 @@ static	char	readbuf[READBUF_SIZE];
  * Try and find the correct name to use with getrlimit() for setting the max.
  * number of files allowed to be open by this process.
  */
-#ifndef _DO_POLL_
+#if ! USE_POLL
 # ifdef RLIMIT_FDMAX
 #  define RLIMIT_FD_MAX   RLIMIT_FDMAX
 # else
@@ -135,7 +89,7 @@ static	char	readbuf[READBUF_SIZE];
 #   endif
 #  endif
 # endif
-#endif /* _DO_POLL_ */
+#endif /* USE_POLL */
 
 /*
 ** add_local_domain()
@@ -191,7 +145,8 @@ aClient *cptr;
 {
 	Reg	int	errtmp = errno; /* debug may change 'errno' */
 	Reg	char	*host;
-	int	err, len = sizeof(err);
+	int	err;
+	SOCK_LEN_TYPE len = sizeof(err);
 	extern	char	*strerror();
 
 	host = (cptr) ? get_client_name(cptr, FALSE) : "";
@@ -230,7 +185,8 @@ char	*ipmask, *ip;
 int	port;
 {
 	static	struct sockaddr_in server;
-	int	ad[4], len = sizeof(server);
+	int	ad[4];
+	SOCK_LEN_TYPE len = sizeof(server);
 	char	ipname[20];
 
 	ad[0] = ad[1] = ad[2] = ad[3] = 0;
@@ -466,17 +422,13 @@ void	close_listeners()
 void	init_sys()
 {
 	Reg	int	fd;
-#ifndef _DO_POLL_
+#if ! USE_POLL
 # ifdef RLIMIT_FD_MAX
 	struct rlimit limit;
 
 	if (!getrlimit(RLIMIT_FD_MAX, &limit))
 	    {
-#  ifdef	pyr
-		if (limit.rlim_cur < MAXCONNECTIONS)
-# else
 		if (limit.rlim_max < MAXCONNECTIONS)
-#  endif
 		    {
 			(void)fprintf(stderr,"ircd fd table too big\n");
 			(void)fprintf(stderr,"Hard Limit: %d IRC max: %d\n",
@@ -484,7 +436,6 @@ void	init_sys()
 			(void)fprintf(stderr,"Fix MAXCONNECTIONS\n");
 			exit(-1);
 		    }
-#  ifndef	pyr
 		limit.rlim_cur = limit.rlim_max; /* make soft limit the max */
 		if (setrlimit(RLIMIT_FD_MAX, &limit) == -1)
 		    {
@@ -492,7 +443,6 @@ void	init_sys()
 					(int) limit.rlim_cur);
 			exit(-1);
 		    }
-#  endif
 	    }
 # endif
 # ifdef sequent
@@ -510,7 +460,7 @@ void	init_sys()
 	    }
 #  endif
 # endif
-#endif /* _DO_POLL_ */
+#endif /* USE_POLL */
 
 #if defined(PCS) || defined(DYNIXPTX) || defined(SVR3)
 	char	logbuf[BUFSIZ];
@@ -608,7 +558,7 @@ Reg	aClient	*cptr;
 Reg	char	*sockn;
 {
 	struct	sockaddr_in sk;
-	int	len = sizeof(struct sockaddr_in);
+	SOCK_LEN_TYPE len = sizeof(struct sockaddr_in);
 
 #ifdef	UNIXPORT
 	if (IsUnixSocket(cptr))
@@ -647,8 +597,8 @@ Reg	char	*sockn;
  * Ordinary client access check. Look for conf lines which have the same
  * status as the flags passed.
  *  0 = Success
- * -1 = Access denied
- * -2 = Bad socket.
+ * -1 = Bad socket.
+ * -2 = Access denied
  */
 int	check_client(cptr)
 Reg	aClient	*cptr;
@@ -662,7 +612,7 @@ Reg	aClient	*cptr;
 		cptr->name, inetntoa((char *)&cptr->ip)));
 
 	if (check_init(cptr, sockname))
-		return -2;
+		return -1;
 
 	if (!IsUnixSocket(cptr))
 		hp = cptr->hostp;
@@ -980,13 +930,16 @@ aClient	*cptr;
 int	hold_server(cptr)
 aClient	*cptr;
 {
+	return -1; /* needs to be fixed, don't forget virtual hosts */
+
+#if 0              /* code and variables declarations are removed, this
+		      avoids compiler warnings */
+
 	struct	sockaddr_in	sin;
 	aConfItem	*aconf;
 	aClient	*acptr;
 	int	fd;
 
-	return -1; /* needs to be fixed, don't forget virtual hosts */
-
 #ifdef	ZIP_LINKS
 	/*
 	 * reconnecting will not work with compressed links,
@@ -1002,9 +955,6 @@ aClient	*cptr;
 	if (!aconf->port)
 		return -1;
 
-	if (cptr->serv->version == SV_OLD)
-		return -1;
-
 	fd = socket(AF_INET, SOCK_STREAM, 0);
 
 	if (fd >= MAXCLIENTS)
@@ -1058,6 +1008,7 @@ aClient	*cptr;
 		acptr, acptr->fd));
 	return 0;
 #endif
+#endif
 }
 
 /*
@@ -1229,12 +1180,12 @@ aClient	*cptr;
 		report_error("setsockopt(SO_REUSEADDR) %s:%s", cptr);
 #endif
 #if  defined(SO_DEBUG) && defined(DEBUGMODE) && 0
-/* Solaris with SO_DEBUG writes to syslog by default */
-#if !defined(SVR4) || defined(USE_SYSLOG)
+/* Solaris 2.x with SO_DEBUG writes to syslog by default */
+#if ! SOLARIS_2 || defined(USE_SYSLOG)
 	opt = 1;
 	if (SETSOCKOPT(fd, SOL_SOCKET, SO_DEBUG, &opt, opt) < 0)
 		report_error("setsockopt(SO_DEBUG) %s:%s", cptr);
-#endif /* SVR4 */
+#endif /* SOLARIS_2 */
 #endif
 #ifdef	SO_USELOOPBACK
 	opt = 1;
@@ -1293,7 +1244,9 @@ aClient	*cptr;
 int	get_sockerr(cptr)
 aClient	*cptr;
 {
-	int errtmp = errno, err = 0, len = sizeof(err);
+	int errtmp = errno, err = 0;
+	SOCK_LEN_TYPE len = sizeof(err);
+
 #ifdef	SO_ERROR
 	if (cptr->fd >= 0)
 		if (!GETSOCKOPT(cptr->fd, SOL_SOCKET, SO_ERROR, &err, &len))
@@ -1323,13 +1276,13 @@ aClient *cptr;
 	**	 as can be seen by the PCS one.  They are *NOT* all the same.
 	**	 Heed this well. - Avalon.
 	*/
-#ifdef	NBLOCK_POSIX
+#if NBLOCK_POSIX
 	nonb |= O_NONBLOCK;
 #endif
-#ifdef	NBLOCK_BSD
+#if NBLOCK_BSD
 	nonb |= O_NDELAY;
 #endif
-#ifdef	NBLOCK_SYSV
+#if NBLOCK_SYSV
 	/* This portion of code might also apply to NeXT.  -LynX */
 	res = 1;
 
@@ -1367,7 +1320,7 @@ aClient *cptr;
 	    {
 		if ((*blscn)->PT+CLONE_PERIOD < timeofday)
 		    {
-			blptr=*blscn;
+			blptr= *blscn;
 			*blscn=blptr->next;
 			MyFree((char *)blptr);
 		    }
@@ -1418,7 +1371,7 @@ int	fd;
 	else
 	    {
 		struct	sockaddr_in addr;
-		int	len = sizeof(struct sockaddr_in);
+		SOCK_LEN_TYPE len = sizeof(struct sockaddr_in);
 
 		if (getpeername(fd, (SAP)&addr, &len) == -1)
 		    {
@@ -1684,7 +1637,7 @@ time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 		*/
 FdAry	*fdp;
 {
-#ifndef _DO_POLL_
+#if ! USE_POLL
 # define SET_READ_EVENT( thisfd )	FD_SET( thisfd, &read_set)
 # define SET_WRITE_EVENT( thisfd )	FD_SET( thisfd, &write_set)
 # define CLR_READ_EVENT( thisfd )	FD_CLR( thisfd, &read_set)
@@ -1729,26 +1682,14 @@ FdAry	*fdp;
 	Reg	aClient	*cptr;
 	Reg	int	nfds;
 	struct	timeval	wait;
-#ifdef	pyr
-	struct	timeval	nowt;
-	u_long	us;
-#endif
 	time_t	delay2 = delay;
 	u_long	usec = 0;
 	int	res, length, fd, i, fdnew;
 	int	auth;
 
-#ifdef NPATH
-         note_delay(&delay);
-#endif
-#ifdef	pyr
-	(void) gettimeofday(&nowt, NULL);
-	timeofday = nowt.tv_sec;
-#endif
-
 	for (res = 0;;)
 	    {
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 		FD_ZERO(&read_set);
 		FD_ZERO(&write_set);
 #else
@@ -1758,12 +1699,12 @@ FdAry	*fdp;
 		pfd->fd  = -1;
 		res_pfd  = NULL;
 		udp_pfd  = NULL;
-#endif	/* _DO_POLL_ */
+#endif	/* USE_POLL */
 		auth = 0;
 
-#ifdef	_DO_POLL_
+#if USE_POLL
 		if ( auth == 0 )
-			bzero((char *) &authclnts, sizeof( authclnts ));
+			bzero((char *) authclnts, sizeof( authclnts ));
 #endif
 		for (i = fdp->highest; i >= 0; i--)
 		    {
@@ -1781,7 +1722,7 @@ FdAry	*fdp;
 					fd));
 				if (cptr->flags & FLAGS_WRAUTH)
 					SET_WRITE_EVENT(cptr->authfd);
-#ifdef	_DO_POLL_
+#if USE_POLL
 				authclnts[cptr->authfd] = cptr;
 #else
 				if (cptr->authfd > highfd)
@@ -1790,7 +1731,7 @@ FdAry	*fdp;
 			    }
 			if (DoingDNS(cptr) || DoingAuth(cptr))
 				continue;
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 			if (fd > highfd)
 				highfd = fd;
 #endif
@@ -1828,30 +1769,13 @@ FdAry	*fdp;
 			    IsReconnect(cptr)
 #endif
 			    ) /* for emacs auto-indentation */
-#ifndef	pyr
 				SET_WRITE_EVENT( fd );
-#else
-			    {
-				if (!(cptr->flags & FLAGS_BLOCKED))
-					SET_WRITE_EVENT( fd );
-				else
-					delay2 = 0, usec = 500000;
-			    }
-			if (timeofday - cptr->lw.tv_sec &&
-			    nowt.tv_usec - cptr->lw.tv_usec < 0)
-				us = 1000000;
-			else
-				us = 0;
-			us += nowt.tv_usec;
-			if (us - cptr->lw.tv_usec > 500000)
-				cptr->flags &= ~FLAGS_BLOCKED;
-#endif
 		    }
 
 		if (udpfd >= 0)
 		    {
 			SET_READ_EVENT(udpfd);
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 			if (udpfd > highfd)
 				highfd = udpfd;
 #else
@@ -1861,7 +1785,7 @@ FdAry	*fdp;
 		if (resfd >= 0)
 		    {
 			SET_READ_EVENT(resfd);
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 			if (resfd > highfd)
 				highfd = resfd;
 #else
@@ -1869,19 +1793,15 @@ FdAry	*fdp;
 #endif			
 		    }
 		Debug((DEBUG_L11, "udpfd %d resfd %d", udpfd, resfd));
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 		Debug((DEBUG_L11, "highfd %d", highfd));
 #endif
 		
 		wait.tv_sec = MIN(delay2, delay);
 		wait.tv_usec = usec;
-#ifndef	_DO_POLL_
-# ifdef	HPUX
-		nfds = select(highfd + 1, (int *)&read_set, (int *)&write_set,
-			      0, &wait);
-# else
-		nfds = select(highfd + 1, &read_set, &write_set, 0, &wait);
-# endif
+#if ! USE_POLL
+		nfds = select(highfd + 1, (SELECT_FDSET_TYPE *)&read_set,
+			      (SELECT_FDSET_TYPE *)&write_set, 0, &wait);
 #else
 		nfds = poll( poll_fdarray, nbr_pfds,
 			     wait.tv_sec * 1000 + wait.tv_usec/1000 );
@@ -1890,7 +1810,7 @@ FdAry	*fdp;
 			return -1;
 		else if (nfds >= 0)
 			break;
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 		report_error("select %s:%s", &me);
 #else
 		report_error("poll %s:%s", &me);
@@ -1903,7 +1823,7 @@ FdAry	*fdp;
 	    } /* for(res=0;;) */
 	
 	if (nfds > 0 &&
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 	    resfd >= 0 &&
 #else
 	    (pfd = res_pfd) &&
@@ -1915,7 +1835,7 @@ FdAry	*fdp;
 		CLR_READ_EVENT(resfd);
 	    }
 	if (nfds > 0 &&
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 	    udpfd >= 0 &&
 #else
 	    (pfd = udp_pfd) &&
@@ -1927,25 +1847,25 @@ FdAry	*fdp;
 		CLR_READ_EVENT(udpfd);
 	    }
 
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 	for (i = fdp->highest; i >= 0; i--)
 #else
 	for (pfd = poll_fdarray, i = 0; i < nbr_pfds; i++, pfd++ )
 #endif
 	    {
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 		if (!(cptr = local[fd = fdp->fd[i]]))
 			continue;
 #else
 		fd = pfd->fd;
-		if (cptr = authclnts[fd])
+		if ((cptr = authclnts[fd]))
 		    {
 #endif
 			/*
 			 * check for the auth fd's
 			 */
 			if (auth > 0 && nfds > 0
-#ifndef	_DO_POLL_
+#if ! USE_POLL
 			    && cptr->authfd >= 0
 #endif
 			    )
@@ -1963,7 +1883,7 @@ FdAry	*fdp;
 				    }
 				continue;
 			    }
-#ifdef	_DO_POLL_
+#if USE_POLL
 		    }
 		fd = pfd->fd;
 		if (!(cptr = local[fd]))
@@ -2002,7 +1922,7 @@ FdAry	*fdp;
 				sendto_flag(SCH_ERROR,
 					    "All connections in use. (%s)",
 					    get_client_name(cptr, TRUE));
-				find_bounce(NULL, fdnew);
+				find_bounce(NULL, 0, fdnew);
 				(void)send(fdnew,
 					   "ERROR :All connections in use\r\n",
 					   32, 0);
@@ -2772,7 +2692,8 @@ static	void	polludp()
 	Reg	char	*s;
 	struct	sockaddr_in	from;
 	Ping	pi;
-	int	n, fromlen = sizeof(from);
+	int	n;
+	SOCK_LEN_TYPE fromlen = sizeof(from);
 
 	/*
 	 * find max length of data area of packet.
